/*
    libmaus2
    Copyright (C) 2019 German Tischler-Höhle

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <libmaus2/parallel/StdMutex.hpp>

libmaus2::parallel::StdMutex::StdMutex() : mutex()
{
}
libmaus2::parallel::StdMutex::~StdMutex() throw()
{
}

void libmaus2::parallel::StdMutex::lock()
{
        while ( ! mutex.try_lock_for(std::chrono::seconds(1)) )
        {

        }
}
void libmaus2::parallel::StdMutex::unlock()
{
        mutex.unlock();
}

bool libmaus2::parallel::StdMutex::trylock(uint64_t const seconds)
{
        return mutex.try_lock_for(std::chrono::seconds(seconds));
}

/**
 * try to lock mutex. returns true if locking was succesful, false if mutex
 * was already locked
 **/
bool libmaus2::parallel::StdMutex::trylock()
{
        return mutex.try_lock();
}

/*
 * try to lock mutex. if succesful, mutex is unlocked and return value is true,
 * otherwise return value is false
 */
bool libmaus2::parallel::StdMutex::tryLockUnlock()
{
        bool const r = trylock();
        if ( r )
                unlock();
        return r;
}

libmaus2::parallel::ScopeStdMutex::ScopeStdMutex(StdMutex & rmutex, bool const prelocked)
: mutex(rmutex)
{
        if ( ! prelocked )
                mutex.lock();
}
libmaus2::parallel::ScopeStdMutex::~ScopeStdMutex()
{
        mutex.trylock();
        mutex.unlock();
}
void libmaus2::parallel::ScopeStdMutex::lock()
{
        mutex.lock();
}

void libmaus2::parallel::ScopeStdMutex::unlock()
{
        mutex.unlock();
}
